@@ -1,6 +1,8 @@
 mod argument;
 mod contract_trait;
 mod endpoint;
+mod endpoint_location;
+mod endpoint_mutability_metadata;
 mod method;
 mod payable;
 mod supertrait;
@@ -8,6 +10,8 @@ mod supertrait;
 pub use argument::*;
 pub use contract_trait::*;
 pub use endpoint::*;
+pub use endpoint_location::*;
+pub use endpoint_mutability_metadata::*;
 pub use method::*;
 pub use payable::*;
 pub use supertrait::*;
